import PySimpleGUI as sg
import global_variables
import get_pdfs
import get_json
import login
import os
import pandas as pd
import requests
import json

CURRENT_DIR = os.path.dirname(__file__)

# This function makes it so when the user selects file paths for the csv, json files, and pdf files,
# their choices will be saved to the global variables in modules/global_variables.py, that way these paths
# can be referenced from anywhere in the program.
def declare_globals(event, values):
    global_variables.CSV_INPUT_PATH = values['pathCSV']
    global_variables.JSON_INPUT_OUTPUT_PATH = values['pathJSON']
    global_variables.PDF_OUTPUT_PATH = values['pathPDF']
    print(f"CSV GLOBAL = {global_variables.CSV_INPUT_PATH}\nJSON GLOBAL = {global_variables.JSON_INPUT_OUTPUT_PATH}\nPDF GLOBAL = {global_variables.PDF_OUTPUT_PATH}")



# The first layout is the screen where you choose your paths and select what files you want to download.
layout = [
    [sg.Txt('Input CSV')],
    [sg.Input(os.path.abspath(os.path.join(CURRENT_DIR,"csv", "input.csv")),size=(50,1), key="pathCSV"), sg.FileBrowse("Browse", key="browseCSV", initial_folder="csv", file_types=(("CSV Files", "*.csv"),))],
    [sg.Txt('JSON Directory')],
    [sg.Input(os.path.abspath("json-output"),size=(50,1), key="pathJSON"), sg.FolderBrowse("Browse", key="browseJSON",initial_folder="json-output")],
    [sg.Txt('PDF Directory')],
    [sg.Input(os.path.abspath("pdf-output"),size=(50,1), key="pathPDF"), sg.FolderBrowse("Browse", key="browsePDF", initial_folder="pdf-output")],
    [sg.Button("Get JSON & PDFs", key="getJSON_PDF"), sg.Button("Get JSON", key="getJSON"), sg.Button("Get PDFs", key="getPDF")],
    [sg.Image("img/spinner.gif", key="spinner")]
]

# This layout is for the login screen and asks the user for their username, password, and client_matter.
loginLayout = [
    [sg.Txt("Log in to Docket Alarm")],
    [sg.Txt("Username:"), sg.Input(size=(50,1), key="username")],
    [sg.Txt("Password:"), sg.Input(size=(50,1), key="password")],
    [sg.Txt("Client Matter:"), sg.Input(size=(50,1), key="clientMatter")],
    [sg.Button("Submit", key="submit")]
]


# This code assigns the layout to their windows. This does not pull up the windows automatically.
# .read() must be called on these to open them. This merely selects the text in the window header for each
# window we might open in the program.
window = sg.Window('Bulk-Docket-Pull', layout)
loginWindow = sg.Window("Log In", loginLayout)


def display_login_window():
    """
    Displays the window to log in to Docket Alarm.
    Prompts the user for their username, password, and client matter.
    """
    while True:
        # Read() opens the window of our choice.
        # Event is the key for the button pressed. We can make logic around this to give buttons their abilities.
        # Values is a dictionary containing any information the user entered into the program.
        event, values = loginWindow.read()

        # An attempt at animating a loading spinner when PDFs are downloading.
        window.Element('spinner').UpdateAnimation("img/spinner.gif",  time_between_frames=50)

        # if the submit button in this window is pressed...
        if event == "submit":

            # We ready up the api endpoint for logging in to Docket Alarm...
            login_url = "https://www.docketalarm.com/api/v1/login/"

            # We ready up the parameters to send to the login endpoint, with the values the user specified...
            data = {
                'username':values['username'],
                'password':values['password'],
                'client_matter':values['clientMatter'],
                }

            # We send the parameters to the API endpoint, storing the resulting json data in a variable...
            result = requests.post(login_url, data=data)

            # We convert the json data from the result of the API call to a python dictionary, making it
            # easier to work with.
            result_json = result.json()

            # We check if the login is a success.
            # If it is not...
            if result_json['success'] != True:
                # We display a popup letting the user know, returning them back to the sign in form.
                sg.popup_error("Invalid Username or Password.")
            else:
                # If it is a success...
                # We save the login info locally to a pickle file so they won't have to log in again next time they use
                # the script.
                login.store_user_info_locally(values['username'], values['password'], values['clientMatter'])
                # We let them know their login was successful.
                sg.popup_ok("Login successful!")
                # We close the login window...
                loginWindow.close()
                # And bring the user to the main window, where they can select their paths, and choose which data to download.
                display_main_window()

def display_main_window():
    """
    Displays the main window of the program.
    This window prompts the user for the paths for their CSV, their PDF files, and their JSON files.
    """
    while True:
        # Read() opens the window of our choice.
        # Event is the key for the button pressed. We can make logic around this to give buttons their abilities.
        # Values is a dictionary containing any information the user entered into the program.
        event, values = window.read()
        print(event, values) # For debugging - Prints buttons pressed, and values returned to the console.
        # If the user selects the 'Get JSON & PDFs' button, we run the function that gets JSON and PDF files.
        if event == "getJSON_PDF":
            # Sets the path choices specified as the global variables that can be used throughout the whole program
            # to reference their choice.
            declare_globals(event, values)
            # Downloads JSON files and PDF files.
            main.get_json_and_pdfs()
        # If the user selects the 'Get JSON' button, we run the function that gets JSON files.
        if event == "getJSON":
            # Sets the path choices specified as the global variables that can be used throughout the whole program
            # to reference their choice.
            declare_globals(event, values)
            # Downloads JSON files.
            get_json.loop_dataframe()
        # If the user selects the 'Get PDFs' button, we run the function that gets the PDF files.
        # (JSON files must be downloaded first to use this option.)
        if event == "getPDF":
            # Sets the path choices specified as the global variables that can be used throughout the whole program
            # to reference their choice.
            declare_globals(event, values)
            # Gets all the links to PDF files from within the json files in the directory the user specified.
            # This is a list of tuples.
            link_list = get_pdfs.get_urls("json-output")
            # Downloads all of the PDF files. Takes the link list from above as an argument.
            get_pdfs.thread_download_pdfs(link_list)
        # If the user closes the window with the "X" in the corner...
        if event == sg.WIN_CLOSED:
            # Close the window.
            break

def gui_run():
    """
    Used for running the GUI.
    Checks to see if valid user login information is stored on the local machine.
    If it is not, it prompts the user to log in.
    If it is, it logs the user in and opens directly to the main window where the
    user can select their filepaths and what files they want to download.
    """
    # If there is no file stored locally containing valid login credentials...
    if not os.path.isfile(os.path.join(CURRENT_DIR, "sav", "credentials.pickle")):
        # Prompt the user to enter their login info.
        display_login_window()
    # If there is a file stored locally containing valid login credentials...
    else:
        # Bring the user to the main window of the program.
        display_main_window()
    
# If this file is run directly...    
if __name__ == "__main__":
    # Crunch the logic to see if the user has logged in successfully before,
    # and open the correct window for them.
    gui_run()


# Built-in Modules
import sys
sys.path.insert(0,".")
import unittest
import os
# Internal Modules
from modules import global_variables, get_json


global_variables.JSON_INPUT_OUTPUT_PATH = global_variables.JSON_INPUT_OUTPUT_PATH

class TestGetJson(unittest.TestCase):

    def test_write_to_json_file(self):
        get_json.write_to_json_file("test", {'test':'yes'})
        bool_ = os.path.isfile(os.path.join(global_variables.JSON_INPUT_OUTPUT_PATH,'test.json'))
        self.assertTrue(bool_)
        os.remove(os.path.join(global_variables.JSON_INPUT_OUTPUT_PATH, 'test.json'))

    def test_authenticate(self):
        result = get_json.authenticate()
        self.assertTrue((len(result) > 10) and (isinstance(result, str)))

    def test_get_docket(self):
        result = get_json.get_docket("17-645","Supreme Court of the United States")
        self.assertTrue(result['success'])

    def test_format_case_number(self):
        result = get_json.format_case_number("16-2013-CF-006932-AXXX-MA")
        self.assertEqual(result, "2013-CF-006932")


if __name__ == '__main__':
    unittest.main()


import os

CURRENT_DIR = os.path.dirname(__file__)
### This file is for storing global variables that can be accessed between modules ###

# The path to the csv file containing the docket numbers
CSV_INPUT_PATH = os.path.join(CURRENT_DIR, "csv")

# The path where json files are saved to, and where the program will look for to find links when downloading pdfs
JSON_INPUT_OUTPUT_PATH = os.path.join(CURRENT_DIR, "json-output")

# The path wherer the folders full of pdfs will be saved to
PDF_OUTPUT_PATH = os.path.join(CURRENT_DIR, "pdf-output")

# The reason for using the script. Used for billing purposes.
CLIENT_MATTER = ""


import sys
sys.path.insert(0,".")
from modules import get_pdfs
import unittest
import os

class TestGetPDFs(unittest.TestCase):

    def test_cleanhtml(self):
        testString1 = "<span> DECLARATION of Matthew Ambros & William Seymour in ... </span>"
        testString2 = "<span>***NOTE TO ATTORNEY TO RE-FILE DOCUMENT - NON-ECF DOCUMENT ERROR. Note to ... </span>"
        testString3 = "<span>Minute Entry for proceedings held before Judge Katherine Polk Failla: ...</span>"
        testString4 = "MOTION_TO_DECLARE_SECTION_921.1417_FLA_STAT_UNCONSTITUTIONAL_AND_REQUEST_FOR_PROFFER_OF_VICTIM_IMPACT_TESTIMONY_AND_PRETRIAL_RULING_ON_WHETHER_THE_DANGER_OF_UNFAIR_PREJUDICE_OF_THAT_EVIDENCE_OUTWEIGHS_ITS_PROBATIVE_VALUE_ANDOR_OTHERWISE_DENIES_A_FAIR_SENTENCING_PROCEEDING_entered_01222020."
        result1 = get_pdfs.cleanhtml(testString1)
        result2 = get_pdfs.cleanhtml(testString2)
        result3 = get_pdfs.cleanhtml(testString3)
        result4 = get_pdfs.cleanhtml(testString4)
        self.assertEqual(result1, "DECLARATION_of_Matthew_Ambros__William_Seymour_in_")
        self.assertEqual(result2, "NOTE_TO_ATTORNEY_TO_RE-FILE_DOCUMENT_-_NON-ECF_DOCUMENT_ERROR_Note_to_")
        self.assertEqual(result3, "Minute_Entry_for_proceedings_held_before_Judge_Katherine_Polk_Failla_")
        self.assertEqual(result4, "MOTION_TO_DECLARE_SECTION_9211417_FLA_STAT_UNCONSTITUTIONAL_AND_REQUEST_FOR_PROFFER_OF_VICTIM_IMPACT_TESTIMONY_AND_PRETRIAL_RULING_ON_WHETHER_THE_DANGER_OF_UNFAIR_PREJUDICE_OF_THAT_EVIDENCE_OUTWEIGHS_ITS_PROBATIVE_VALUE_ANDOR_OTHERWISE_DENI")

    def test_download_from_link_list(self):
        get_pdfs.download_from_link_list("https://www.nasa.gov/pdf/390539main_Athlete%20Fact%20Sheet.pdf","test_pdf","pdf_test", "test")
        bool_ = os.path.isfile(os.path.join('test',"pdf_test",'test_pdf.pdf'))
        self.assertTrue(bool_)
        os.remove(os.path.join("test","pdf_test", 'test_pdf.pdf'))
        os.rmdir(os.path.join("test","pdf_test"))

    def test_add_path_to_list_of_tuples(self):
        testInput = [("eggs", "milk"), ("onions", "flour"), ("toothpaste", "orange juice")]
        expectedResult = [("eggs", "milk", "sand"), ("onions", "flour","sand"), ("toothpaste", "orange juice", "sand")]
        actualResult = get_pdfs.add_path_to_list_of_tuples(testInput, "sand")
        self.assertEqual(expectedResult,actualResult)

if __name__ == '__main__':
    unittest.main()



# Built-in Modules
import re           # For pattern matching within strings
import json         # For working with json data
import os           # For accessing features of the operating system
import threading
import concurrent.futures
import datetime
import time
# Third-party Modules
from progress.bar import IncrementalBar    # For showing graphical loading bars
from retrying import retry                 # For automatically retrying code that throws errors
import requests                            # For making http requests
import pandas as pd                        # For working with tabular data
from tqdm import tqdm
# Internal Modules
import config
import login, file_browser, global_variables
import gui #DEV
import PySimpleGUI as sg

CURRENT_DIR = os.path.dirname(__file__)

# Reinforces that the variables defined in the global_variables module, and then edited from within other modules,
# continue to have the value that the user changed it to.
# It may look redundant, but without this line, the script only uses the default variable, without reflecting changes.

global_variables.JSON_INPUT_OUTPUT_PATH = global_variables.JSON_INPUT_OUTPUT_PATH
global_variables.CLIENT_MATTER = global_variables.CLIENT_MATTER

# We create an instance of a lock object. We use this later when we download json with threads, to ensure that no 
# two threads try to access data in the same place at the same time, causing problems.
lock = threading.Lock()


def table_to_list_of_tuples():
    """
    Grabs the csv from the CSV_INPUT_PATH variable that the user specified in the main menu,
    and returns a list of tuples. Each tuple in the list is a set of arguments ready to be passed to
    the download_json_from_list_of_tuples() function within the thread_download_json() function
    that wraps both of these funtions to use threading to download more quickly.
    """

    # The path to the input spreadsheet is the path that the user specified in the main menu.
    spreadsheet_path = global_variables.CSV_INPUT_PATH

    # The path where the JSON files will be downloaded to is the path that the user specified in the main menu.
    JSON_INPUT_OUTPUT_PATH = global_variables.JSON_INPUT_OUTPUT_PATH

    # The client matter is the string that the user specified in the main menu.
    CLIENT_MATTER = global_variables.CLIENT_MATTER

    # This list starts out empty, gets a tuple appended to it with every iteration of the loop below, and will eventually
    # be the value returned by this function.
    output_list_of_tuples = []

    try:
        # We try to open the csv as a pandas dataframe. Pandas dataframes make working with tabular data in python faster and easier.
        df = pd.read_csv(spreadsheet_path)

    except Exception as e:
        # If there are any errors with opening the dataframe, we print the data to the console to alert the user.
        print(f"{e}")
        input()
    
    # We loop through every row of the input spreadsheet, the row value allows us to access each value in each row through indexing.
    for index, row in df.iterrows():
        # We use indexing to store each value in the appropriate variables so they are more human-readable.
        caseName = row[0]
        caseNo = row[1]
        caseCourt = row[2]
        # We place the values into a tuple that will serve as parameters for download_json_from_list_of_tuples()
        # when we call it inside the thread_download_json() wrapper.
        row_tuple = (caseName, caseNo, caseCourt, JSON_INPUT_OUTPUT_PATH, CLIENT_MATTER)
        # We append each tuple to the list at the top of the function.
        output_list_of_tuples.append(row_tuple)
    # We return the list after it is populated with tuples during each iteration over every row in the spreadsheet.
    return output_list_of_tuples

@retry
def download_json_from_list_of_tuples(result_tuple):
    """
    This function takes in a tuple with 5 arguments as strings in order:
    The case name,
    The case number,
    The court the case is in,
    The output path for the download,
    The client matter (The reason for making the call, for billing purposes).
    It downloads json data for each case.
    This function is not called on its own, it is wrapped by the 
    thread_download_json() function, which allows each call of the function to be done in it's
    own thread, speeding up the download.
    """

    # We unpack the tuple and assign all of it's values to human-readable variable names.
    caseName, caseNo, caseCourt, JSON_INPUT_OUTPUT_PATH, CLIENT_MATTER = result_tuple

    user = login.Credentials()

    # The endpoint we will be connecting to. Calls to this endpoint return the json data for the docket we want.
    getdocket_url = "https://www.docketalarm.com/api/v1/getdocket/"

    # The parameters we pass to the endpoint. This is how the API knows how to find what we are looking for.
    data = {
        # The token generated after logging in.
        'login_token': user.authenticate(),
        # The reason for use
        'client_matter': CLIENT_MATTER,
        # The court we want to search.
        'court': caseCourt,
        # The docket number we want data for
        'docket':caseNo,
        # A boolean representing whether or not we want the cached version of the data.
        'cached': config.isCached,
        # Cleans up names
        'normalize':True,
    }

    # Makes the api call. We specify the endpoint and the parameters as arguments. The results of the API call are returned and
    # stored to a variable. 
    result = requests.get(getdocket_url, data)

    

    try:
        # if the api call fails, a detailed error is thrown. The script does not stop and the error message is not immediately shown to the user.
        result.raise_for_status() 
    except:
        # Rather, the error is written to log/log.txt with a timestamp and information about which case could not be downloaded.
        result_json = None
        print(result)
        print(caseName)
        print(caseNo)
        timeNow = datetime.datetime.now().strftime("%I:%M%p %B %d, %Y")
        with open(os.path.join(CURRENT_DIR, 'log', 'log.txt'), 'a') as errorlog:
            errorlog.write(f"\n{timeNow}\n")
            errorlog.write("JSON could not be downloaded:\n")
            errorlog.write(f"{result}: {caseName}, {caseNo}, {caseCourt}\n")
            errorlog.write("------------------")
        return

    result_json = result.json()
    # We use .json() to convert the json results to a python dictionary we can more easily work with.

    
    try:
        # Creates the path where our .json file will be saved to
        filePathNameWExt = os.path.join(JSON_INPUT_OUTPUT_PATH, f"{caseName} {caseNo}" + '.json')


        # We use a lock so this code won't be executed by multiple threads simultaneously, this way we don't get errors.
        with lock:
            # When 'opening' a file that doesn't yet exist, we create that file.
            # Here, we create the json file we'll be saving the data to.
            with open(filePathNameWExt, 'w') as fp:

                # Then we write the data to the newly created .json file.
                json.dump(result_json,fp, indent=3)

    # If the api call was successful, but the writing of the data to a file fails, we display the error message to the user.
    except Exception as e:
        print("\nError writing json file.\nReference the documentation for more information\n")
        input()
        print(e)
    return

def thread_download_json():
    """
    Wrapper function for download_json_from_list_of_tuples
    and table_to_list_of_tuples().
    table_to_list_of_tuples() is called, and it's return values are stored in a variable and passed as arguments
    to individual calls of download_json_from_list_of_tuples() within individual threads, speeding up the download.
    """

    # We call table_to_list_of_tuples() and store the results in a variable.
    tuples_from_table = table_to_list_of_tuples()
    # We get the amount of iterations the program will make, this will be used to tell the loading bar when it will be done.
    maximum = len(tuples_from_table)
    print("Downloading JSON files...")
    # We start a counter, so at the end we can calculate how long the downloads took.
    start = time.perf_counter()
    with concurrent.futures.ThreadPoolExecutor() as executor:
    # We start concurrent.futures to have every line of code within the block get passed to its own sepreate thread.
        results = list(tqdm(executor.map(download_json_from_list_of_tuples, tuples_from_table), total=maximum))
        # We use executor.map to use threading, it takes the function and a list of arguments to pass as arguments.
        # tdqm starts a progress bar, and we specify the max value it needs to reach to finish.
    # We store the time again when it is over.
    finish = time.perf_counter()
    # We subtract the start time from the finish time to let the user know how long the download took.
    print(f"Finished downloading JSON files in {round(finish-start)} seconds.")
    try:
        # If the users operating system permits, we open the download directory where the desired output files were downloaded to.
        os.startfile(global_variables.JSON_INPUT_OUTPUT_PATH)
    except:
        pass
        
    return results


# Built-in Modules
import os
import json
from multiprocessing import Pool, cpu_count
import re
import datetime
import threading
import time
import concurrent.futures
# Third-party Modules
from progress.bar import IncrementalBar
from tqdm import tqdm
import requests
import PyPDF2 #DEV
from retrying import retry
# Internal Modules
import config
import file_browser, global_variables
import gui #DEV
import log_errors_to_table
import login

CURRENT_DIR = os.path.dirname(__file__)

# Reinforces that the variables defined in the global_variables module, and then edited from within other modules,
# continue to have the value that the user changed it to.
# It may look redundant, but without this line, the script only uses the default variable, without reflecting changes.
global_variables.PDF_OUTPUT_PATH = global_variables.PDF_OUTPUT_PATH
global_variables.CLIENT_MATTER = global_variables.CLIENT_MATTER

# We create an instance of a lock object. We use this later when we download json with threads, to ensure that no 
# two threads try to access data in the same place at the same time, causing problems.
lock = threading.Lock()

# We create an ErrorTable object where we can write errors to an xlsx file as they come and then save the file at the end of the download.
tableErrorLog = log_errors_to_table.ErrorTable()

def cleanhtml(raw_html):
    """
    This function is for creating filenames from the HTML returned from the API call.
    It takes in a string containing HTML tags as an argument, and returns a string without HTML tags or any
    characters that can't be used in a filename.
    """

    # Created a regular expression pattern object for detecting HTML tags
    cleanr = re.compile('<.*?>')

    # Removes HTML tags from the string argument
    cleantext = re.sub(cleanr, '', raw_html)

    # Replaces spaces with underscores
    cleantext = str(cleantext).strip().replace(' ', '_')

    # Removes characters that can't be used in filenames
    cleantext = re.sub(r'(?u)[^-\w.]', '', cleantext)

    # Removes periods
    cleantext = cleantext.replace(".", "")

    # Cuts the name short so it doesn't go over the maximum amount of characters for a NTFS filename
    cleantext = cleantext[0:240]

    # Return the text free of html tags and symbols that can't be used in filenames.
    # It will also be cut to a length that doesn't go over the maximum length for a filename. 
    return cleantext

def get_urls(input_directory):
    """
    Takes in a directory full of JSON files as input, and returns the values for keys labeled 'link' for all of the files.
    The output is a list of tuples.
    The first item in each tuple is a string containing the link.
    The second item in each tuple is a string containing the name of the document the link is connected to.
    The third item in each tuple is a string containing the original file name of the json file that the link was retrieved from.
    """

    # This list will be appended throughout the function with every pdf link found within the json files.
    # It will ultimately be returned at the end of the function.
    pdf_list = []

    # The absolute path of the 'result' folder
    input_directory = global_variables.JSON_INPUT_OUTPUT_PATH

    # The client matter the user specified in the menus.
    CLIENT_MATTER = global_variables.CLIENT_MATTER

    if os.path.isdir(input_directory) == False:
        print("[ERROR] Could not write PDF files.\nMake sure 'json-output' folder exists in the root directroy of the program.\nCheck documentation for more information.\n")
        input()


    # Loops through every file in the 'result' directory.
    for file in os.listdir(input_directory):

        # We store our output path for PDFs from our global variable in a local variable.
        # The output path must be included in the result because when the resulting tuples get passed to our
        # download_from_link_list() function within the thread_download_pdfs() wrapper function, it can't access the 
        # global variable directly from inside the seperate threads.
        PDF_OUTPUT_PATH = global_variables.PDF_OUTPUT_PATH

        # Saves the name of each file in the folder to a variable
        filename = os.fsdecode(file)

        # Checks to ensure that we only run our code on .JSON files
        if not filename.lower().endswith(".json"):
            continue
        # Stores the absolute path of the current JSON file in the loop as a variable. 
        path = os.path.join(input_directory, filename)

        # Designates the extenstion '.json' as a regex pattern that we want to remove from a string.
        text_to_remove = re.compile(re.escape('.json'), re.IGNORECASE)

        # Uses the regex above to remove '.json' from the json filenames, which we will use to name the folders that
        # will contain the corresponding pdfs to the original json file.
        base_filename = text_to_remove.sub("", filename)
        
        # Opens each individual JSON file
        with open(path) as jsonFile:

            # Allows us to work with JSON files the same way we would work with a Python dictionary.
            jsonObject = json.load(jsonFile)
            
            # Checks to see if a 'docket_report' key exists in the current JSON file in the loop.
            if "docket_report" in jsonObject:

                # If it exists, it saves the value for the docket_report key in a variable.
                docket_report = jsonObject['docket_report']

                # docket_report will be a list of dictionaries. This loops through each dictionary in the list.
                for item in docket_report:
                    
                    docName = item['contents']

                    # We run the cleanhtml() function on the document name to remove the HTML tags and acharcters that can't be used in filenames
                    docName = cleanhtml(docName)

                    # The ID number of the document. We use this later for the file names
                    docNum = item['number']

                    # Checks to see if any of the dictionaries inside the list contain a 'link' key
                    if 'link' in item:

                        # The 'link' key contains a link to a PDF file associated with that item in the docket report.
                        link = item['link']

                        link_filename = f"{docNum} - {docName}"

                        link_tuple = (link, link_filename, base_filename, PDF_OUTPUT_PATH, CLIENT_MATTER)
                        # Add the found link to the list, which will ultimately be returned at the end of the function.
                        pdf_list.append(link_tuple)

                    # Some PDF's are inside the exhibits key, which doesnt always exist. Here, we check to see if the exhibits key exists.
                    if 'exhibits' in item:

                        # if it does exist, we save its contents in an exhibits variable.
                        exhibits = item['exhibits']
                        
                        # The data contained inside 'exhibits' will be a list of dictionaries. So we loop through the list to access the data.
                        for exhibit in exhibits:

                            # We chck to see if any links exist inside exhibits
                            if 'link' in exhibit:

                                exhibitNumber = f"{exhibit['exhibit']}"

                                # If a link to a PDF does exist, we store it in a variable.
                                exhibitLink = exhibit['link']
                                
                                # We create a file name to save the exhibit pdf as
                                exhibitName = f"Exhibit {exhibitNumber} - {docNum} - {docName}"

                                # We package the name, link, and filename together in a tuple, that will be passed as an argument to our
                                # download_from_link_list() function within the thread_download_pdfs() function where we use map to
                                # downloading with seperate threads, speeding things up.
                                exhibitLink_tuple = (exhibitLink, exhibitName, base_filename, PDF_OUTPUT_PATH, CLIENT_MATTER)
                                pdf_list.append(exhibitLink_tuple)

            # We close the file when we are done. This also ensures that the file is saved.    
            jsonFile.close()
    return pdf_list

@retry
def download_from_link_list(link_list):
    """
    Downloads PDF documents from the web and saves them in a specified folder.
    Takes in 3 string arguments:
    1. Link to a pdf file
    2. File name we will save as
    3. Name of the folder we will create to store our PDFs.
    Notice how the arguments are the same as what the get_urls() function returns.
    This function Isn't made to be used on its own, but can be.
    """

    # We store a user object we can use to login
    user = login.Credentials()

    # We unpack the tuple, assigning each value to a human-readable variable name.
    link, fileName, folderName, outputPath, CLIENT_MATTER = link_list

    # The directory where we will create the subdirectories within for each individual docket
    outputDirectoryPath = os.path.join(outputPath, folderName)
    # The path we are saving the file to, inside the subdirectory we will create.
    outputFilePath = os.path.join(outputDirectoryPath, f"{fileName}.pdf")
    
    # We open a lock so threads can't run this block of code simultaneously since that would cause errors
    with lock: 
        # If the directory for the docket doesn't yet exist...
        if not os.path.exists(outputDirectoryPath):

            # Then, create it!
            os.makedirs(outputDirectoryPath)
    
    
    # We ready our authentication token to pass as a paramater with our http request to get the pdf file. You must be logged in to access the files.
    params = {
            "login_token": user.authenticate(),
            "client_matter": CLIENT_MATTER,
            }

    # We then make an http request to the pdf link and save the result in a variable. We pass the authentication token as a parameter.
    result = requests.get(link, stream=True, params=params)

    try:
        # If the http request failed, we have it throw a detailed error message. This is not immediately shown to the user and we let the donwload
        # continue for now.
        result.raise_for_status()
    
    except Exception as a:
        timeNow = datetime.datetime.now().strftime("%I:%M%p %B %d, %Y")
        with lock:
            # We write the error to log/log.txt with a timestamp and detailed information about which case caused the error.
            with open(os.path.join(CURRENT_DIR, 'log', 'log.txt'), 'a') as errorlog:
                errorlog.write(f"\n{timeNow}\n")
                errorlog.write(f"{a}")
                errorlog.write(f"\n{link}\n{fileName}\n{folderName}\n{outputPath}\n------------------")
        
            # We write the error to a csv file that will be stored in the log folder when the download finishes.
            tableErrorLog.append_error_table(f"{a}", folderName, fileName)


    try:
        # Once the folder is created, we can create a file inside it, open it, and...
        with open(outputFilePath, "wb") as e:

            # Write the contents of the PDF to the place we specify.
            e.write(result.content)
    
    except Exception as a:
        print(a)
    
    return


def thread_download_pdfs(link_list):
    """
    Wrapper of download_from_link_list()
    Takes in a link_list generated by the get_urls() function.
    """

    # Gets the amount of links that will be downloaded. We use this later because the progress bar takes the maximum
    # amount of downloads as a parameter
    maximum = len(link_list)

    print("Downloading PDF files...")

    # Starts a timer, we end the timer after we run the function with threading to see how long the bulk download
    # took in total.
    start = time.perf_counter()
    # We start up the threading executor
    with concurrent.futures.ThreadPoolExecutor() as executor:
        try:
            # We use executor.map() to select our function and the arguments that will be passed to it in each new thread.
            # We wrap this in list(tdqm()) to add the progress bar. See stackoverflow page below for more info.
            # https://stackoverflow.com/questions/51601756/use-tqdm-with-concurrent-futures
            results = list(tqdm(executor.map(download_from_link_list, link_list), total=maximum))
        except FileExistsError as fee:
            # If we get a FileExistsError, we let the user know that the directory they save to must be empty.
            print("[ERROR] Directory you're saving PDFs to must be empty.")
            input()
            # After pressig enter, we print the error thrown out to the user.
            print(fee)
    # We finish our timer.
    finish = time.perf_counter()
    # We display the amount of time the downloads took all together.
    print(f"Finished downloading PDF files in {round(finish - start)} seconds.")
    # We save the current date and time in a variable
    currentDateTime = datetime.datetime.now().strftime("%I%M%p %B %d, %Y")
    # We save our csv log that has been tracking any errors throughout the downloads.
    # If any PDFs will not open, then they wil be displayed in this PDF.
    # The file will be in the log folder and will be named according to the date and time when
    # the download finished.
    tableErrorLog.error_excel_save(os.path.join(CURRENT_DIR, "log", f"logTable - {currentDateTime}.xlsx"))
    # We must return results to make the progress bar work.
    try:
        os.startfile(global_variables.PDF_OUTPUT_PATH)
    except:
        pass
        
    return results








        
        



# Built-in Modules
import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
# Third-party Modules
from colorama import init, Fore, Back, Style
# Internal Modules
import get_json, get_pdfs, login, menus, file_browser, global_variables, fetch_updated_court_list
import config
import gui

# Inititalizes Colorama functionality, allowing us to write text to the terminal in different colors.
init()

CURRENT_DIR = os.path.dirname(__file__)

msg = Fore.YELLOW + """
  _____             _        _              _                                 _____ _____ 
 |  __ \           | |      | |       /\   | |                          /\   |  __ \_   _|
 | |  | | ___   ___| | _____| |_     /  \  | | __ _ _ __ _ __ ___      /  \  | |__) || |  
 | |  | |/ _ \ / __| |/ / _ \ __|   / /\ \ | |/ _` | '__| '_ ` _ \    / /\ \ |  ___/ | |  
 | |__| | (_) | (__|   <  __/ |_   / ____ \| | (_| | |  | | | | | |  / ____ \| |    _| |_ 
 |_____/ \___/_\___|_|\_\___|\__| /_/    \_\_|\__,_|_|  |_| |_| |_| /_/    \_\_|   |_____|
 |  _ \      | | |    |  __ \                    | |               | |                    
 | |_) |_   _| | | __ | |  | | _____      ___ __ | | ___   __ _  __| |                    
 |  _ <| | | | | |/ / | |  | |/ _ \ \ /\ / / '_ \| |/ _ \ / _` |/ _` |                    
 | |_) | |_| | |   <  | |__| | (_) \ V  V /| | | | | (_) | (_| | (_| |                    
 |____/ \__,_|_|_|\_\ |_____/ \___/ \_/\_/ |_| |_|_|\___/ \__,_|\__,_|                    
                                                                                          
"""

feesNotice = """
[Document Fees]

Courts may charge a fee to access certain documents. Docket Alarm passes their fee on
without markup. After the first access, later access does not incur a fee.
Contact Docket Alarm Support for information on specific fees associated with
particular courts.

Press ENTER to continue. 
"""
def clear():
    """clears the terminal"""
    os.system('cls' if os.name == 'nt' else 'clear')

def welcome():
    """
    The code that executes at launch. It guides the user through menus and starts other functions from the other folders
    based on the users choices.
    """
    clear()
    print(msg)
    print(Style.RESET_ALL)
    print("\n")
    print("Welcome!")
    input("Press ENTER key to begin!")
    clear()
    print(feesNotice)
    input()
    clear()

    # Checks to see if the account information was stored on the local machine previously
    if not os.path.isfile(os.path.join(CURRENT_DIR, "sav", "credentials.pickle")):

        # If the user hasn't successfullly logged in before, it takes them to a menu sequence to log in, and saves the info locally
        # for the next time the script is run.
        login.login_interface()

    
    user = login.Credentials()

    print(f"You are logged in as: {user.username}")
    instructions = """
Instructions:

This program takes in a csv file full of docket numbers and will automatically
populate 2 folders with the raw JSON data and all of the PDF documents associated
with that docket.

You will now select the path where your input csv is located.
Press ENTER to open the file browser.
    """
    print(instructions)
    input()
    clear()

    # Opens a graphical file browser and returns the path to the csv file that the user selected.
    csvChoice = file_browser.browseCSVFiles()

    # Assigns the choice to a global variable, so other modules can find the path that the user specified.
    global_variables.CSV_INPUT_PATH = csvChoice

    options = """
Type in one of the following numbers and press ENTER to specify your choice:

[1] Get all JSON files and PDF files.

[2] Get JSON files only.

[3] Get PDF files only.
    ( Only select 3 if you already have a directory full of JSON files. )
    ( The JSON files are needed to extract the download links from.     )

[4] More options.

Enter your response below.[1/2/3/4]
    """
    print(options)

    def handle_input():
        """
        Prompts the user for a choice and calls the function from the 'modules' folder that corresponds
        with that choice.
        """
        userChoice = input()

        # Choice 1 is downloading all json and pdf files.
        if userChoice == "1":
            clear()
            menus.select_paths_menu()
            clear()
            menus.specify_client_matter_menu()
            print(msg)
            get_json_and_pdfs()
        # Choice 2 is donwloading only JSON files.
        elif userChoice == "2":
            clear()
            menus.select_paths_menu(pdfOption=False)
            menus.specify_client_matter_menu()
            print(msg)
            get_json.thread_download_json()
        # Choice 3 is downloading only PDF files.
        elif userChoice == "3":
            clear()
            menus.select_paths_menu()
            menus.specify_client_matter_menu()
            print(msg)
            link_list = get_pdfs.get_urls("json-output")

            # get_pdfs.multiprocess_download_pdfs(link_list)
            get_pdfs.thread_download_pdfs(link_list)
        elif userChoice == "4":
            clear()
            menus.other_options_menu()

        # If the user enters anything other than a valid choice, then it tells them their choice is invalid and
        # restarts this function, prompting them to make a choice again.
        else:
            print("Please Enter Valid input (1, 2 or 3)")
            return handle_input()

    handle_input()
    try:
        os.startfile(os.path.join(CURRENT_DIR, "log"))
    except:
        pass
        
    print("\nDone.")
    input()



def get_json_and_pdfs():
    """
    This function calls the function to download JSON first, immediately followed by the function to download
    PDFs. This is used when the user chooses the menu choice to download both.
    """

    # The function that downloads the JSON files
    # get_json.loop_dataframe()
    get_json.thread_download_json()

    # The function that extracts the proper arguments to pass to the function for downloading PDFs using multiprocessing.
    # That function requires a list of tuples, each tuple being a seperate set of arguments to pass.
    link_list = get_pdfs.get_urls("json-output")

    # This function uses threading on the function that downloads PDFs, allowing us to download multiple PDFs at once,
    # speeding up the process.
    get_pdfs.thread_download_pdfs(link_list)

def select_paths_menu(pdfOption=True):
    """
    This displays a menu to the user allowing them to choose file paths to save their downloaded data to.
    It handles their choices and changes the global variables representing the file paths for saving.
    """
    print("Below are the current settings for finding and saving files:\n")
    print(f"[1] The CSV file filled with docket numbers is located at:\n{os.path.abspath(global_variables.CSV_INPUT_PATH)}\n")
    print(f"[2] JSON Files are saved to and retrieved from:\n{os.path.abspath(global_variables.JSON_INPUT_OUTPUT_PATH)}\n")
    if pdfOption == True:
        print(f"[3] PDF Files are saved to:\n{os.path.abspath(global_variables.PDF_OUTPUT_PATH)}\n")
    print("To change these paths. Type the number for the path you want to change and press ENTER.\n")
    print(Fore.RED + "[WARNING] A lot of files can be generated in the PDF and JSON directories you choose! Choose carefully!\n" + Style.RESET_ALL)
    print("If you are happy with the current selection, simply press ENTER.\n")

    # Prompts the user for a choice and saves it in a variable.
    userChoice = input()

    # Choice 1 is to edit the path to the input csv full of docket numbers.
    if userChoice == "1":
        print("Select the path of your input csv file.")
        print("Press ENTER to open the file browser.")
        input()
        
        # Opens the file browser and returns the path to the file that the user selected.
        csvChoice = file_browser.browseCSVFiles()

        # We store this choice to a global variable to be used elsewhere in the script where
        # we need to access this choice.
        global_variables.CSV_INPUT_PATH = csvChoice

        clear()

        # Reloads path select menu, relflecting any changes.
        select_paths_menu()

    # Choice 2 is to edit the path to the json files.
    if userChoice == "2":
        print("Select the path where you would like to store your JSON files.")
        print("Press ENTER to open the file browser.")
        input()

        # Stores the users choice in a variable
        jsonChoice = file_browser.browseDirectories('json-output')

        # Stores the choice to a global variable so this choice can be used throughout this script,
        # not only in the context of this file.
        global_variables.JSON_INPUT_OUTPUT_PATH = jsonChoice

        clear()
        # Reloads path select menu, relflecting any changes.
        select_paths_menu()

    # Choice 3 is to edit the path where the folders full of PDF files will be saved.
    if userChoice == "3":
        print("Select the path where you would like to store your PDF files.")
        print("Press ENTER to open the file browser.")
        input()

        # Opens a file explorer and returns the path to the directory the user selected as a string.
        pdfChoice = file_browser.browseDirectories('pdf-output')

        # Saves the chosen file path to a global variable so it can be accessed elsewhere in the script when
        # we need to access this path.
        global_variables.PDF_OUTPUT_PATH = pdfChoice

        clear()
        # Reloads path select menu, relflecting any changes.
        select_paths_menu()

    # If the user doesnt make a choice and just presses ENTER, the program exits this menu and moves forward
    else:
        clear()

def other_options_menu():
    other_options = """
Type in one of the following numbers and press ENTER to specify your choice:

[0] Return to menu.

[1] Fetch updated list of compatible courts.
    (You use these names in your input CSV.                                          )
    (The list is always being updated and changes are not immediately added to Github)

[2] Log Out of Docket Alarm.

Enter your response below.[0/1/2]
    """
    print(other_options)
    userChoice = input()
    if userChoice == "0":
        clear()
        welcome()
    elif userChoice == "1":
        clear()
        print("Fetching updated court list...")
        fetch_updated_court_list.fetch_updated_court_list()
        print("Done.")
        input()
        welcome()
    elif userChoice == "2":
        clear()
        user = login.Credentials()
        print("\nAre you sure you want to log out of Docket Alarm?[Y/n]")
        userChoice = input()
        if userChoice.upper() == "Y":
            user.logout()
            clear()
            welcome()
        else:
            clear()
            other_options_menu()
    else:
        print("Please Enter Valid input (0, 1 or 2)")

def specify_client_matter_menu():
    clear()
    print("Please enter the client or matter code used to bill this search. Max length is 50 characters.\n(If unsure or not applicable, leave blank and press ENTER.)")
    user_input = input()
    global_variables.CLIENT_MATTER = user_input
    clear()
    return




import pandas as pd

class ErrorTable:
    # Used for creating Table (Excel) Error logs.
    # Calling object.df will produce the pandas dataframe.
    # Calling object.append_error_csv(Three string arguments) will add the values to the csv log object.
    # Calling error_csv_save(path) will save the table as a csv to the path you specify.
    def __init__(self):
        """
        Initializes an empty dataframe with three column headers
        """
        self.df = pd.DataFrame(columns=['error', 'docket','document'])

    def __repr__(self):
        # returns the dataframe, using dropna() to remove any null values.
        # Null values usually get added to the class as a glitch.
        return pd.dropna(self.df)

    def append_error_table(self, error, docket, document):
        """
        Takes 3 string arguments, adds them to the csv log in order
        """
        # Before adding a new row, the row must be structured as a dictionary.
        # We create a dictionary whenever this method is called, that has the column labels as keys,
        # and will fill in the values with the arguments passed to the method.
        new_row = {
            "error":error,
            "docket":docket,
            "document":document,
        }
        # Then we use append() to add the new values to the dataframe.
        appender = self.df.append(new_row, ignore_index=True)
        # Append() usually returns a new dataframe with the changes added, rather than actually 
        # changing the original, so we make self.df equal to the changed version.
        self.df = appender.dropna()

    def error_csv_save(self,path):
        """
        Saves the csv when you are done appending it.
        Must specify a path as a string for the argument.
        """
        self.df.to_csv(path, index=False)

    def error_excel_save(self,path):
        """
        Saves the xlsx file when you are done appending it.
        Must specify a file path as a string for the argument.
        """
        self.df.to_excel(path,index=False)




import requests
import os
import login
import global_variables

CURRENT_DIR = os.path.dirname(__file__)

def fetch_updated_court_list():
    """
    Prints all the courts to the console and returns a list of courts
    """

    user = login.Credentials()

    # Sending a get request to the /searchdirect/ endpoint with only the login_token and
    # client matter will return a list of couthouses. This list is every court we can search,
    # with the name formatted in a way that we can send later when accessing the API for 
    # searching for dockets.
    searchdirect_url = "https://www.docketalarm.com/api/v1/searchdirect/"

    data = {
        'login_token':user.authenticate(),
        'client_matter': global_variables.CLIENT_MATTER,
    }

    # returns a json object
    result = requests.get(searchdirect_url, data)


    # We call the .json() method on the json object to turn it into a python dictionary
    result_json = result.json()

    # The list of courts is stored in the 'courts' key we assign to a variable here
    courts = result_json['courts']

    # Designates the file path where the new list of courts will be stored
    updated_courts_output_file = os.path.join(CURRENT_DIR,"docs", "updated-courts.txt")

    # Deletes an old copy of this file if one exists
    try:
        os.remove(updated_courts_output_file)
    except:
        pass

    # Loops through every court in the list
    for court in courts:
        # Opens the output txt file
        with open(updated_courts_output_file, "a") as txt:
            # And writes that court to the file, until all courts are written.
            txt.write(court + "\n")
    
    # Opens the new list of courts on the users desktop
    try:
        os.startfile(updated_courts_output_file)
    except:
        pass
        


    # The function call returns a list object with all the courts we can search with Docket Alarm
    return courts


# Built-in Modules
import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
import pickle
# Third-party Modules
import stdiomask
# Internal Modules
import requests
import menus

CURRENT_DIR = os.path.dirname(__file__)

def store_user_info_locally(username, password):
    """
    Takes in 3 strings as arguments, a username, a password, and the client matter.
    Saves them as a dictionary in a .pickle file in the 'sav' folder to be used later.
    This is so the user can avoid logging in every time they run the script
    """

    # We store the arguments in a dictionary with proper keys to access them later in the script.
    credentials_dict = {
        'username':username,
        'password':password,
    }

    # We choose a file location to save the dictionary to.
    output_path = os.path.join(CURRENT_DIR, "sav", "credentials.pickle")
    # We open a .pickle file to save the dictionary in, which can store python variables to be accessed later.
    # This allows the program to not have to ask the user to log in every time they use the script.
    pickle_out = open(output_path, "wb")

    # We write the dictionary to the pickle file at sav/credentials.pickle
    pickle.dump(credentials_dict, pickle_out)

    # When we are done writing to the file, we close it.
    pickle_out.close()

class Credentials:
    # The allows us to create a Credentials object.
    # We can reference the username, password, and client_matter attributes to get the information we need about the user
    # elsewhere in the script.

    # This is the code that is run when we initialize a new instance of this object
    def __init__(self):
        # We get the path to the .pickle file the user credentials information is stored in.
        input_path = os.path.join(CURRENT_DIR, "sav", "credentials.pickle")
        # We open it up...
        pickle_in = open(input_path, "rb")
        # We store the dictionary inside of it to a variable...
        credentials_dict = pickle.load(pickle_in)
        # And we close the file when we're done making changes.
        pickle_in.close()
        # by setting the below attributes, we can reference the username by calling credentials_object.username, and so on.
        self.username = credentials_dict['username']
        self.password = credentials_dict['password']

    def authenticate(self):
        # """Returns the authentication token to make API calls. Make sure that auth.py is filled out!"""
        # This is the endpoint for logging in to Docket Alarm from the API.
        login_url = "https://www.docketalarm.com/api/v1/login/"

        # The data we will send to the endpoint with our post request will be
        # our Docket Alarm username and password.
        data = {
            'username': self.username,
            'password': self.password,
            }

        # We save the response to a variable. The response is a json object containing
        # our authentication key iside the json key named 'login_token'
        result = requests.post(login_url, data=data)

        result.raise_for_status()
        # Calling the .json() method on the result turns it into a python dictionary we
        # can work with natively.
        result_json = result.json()

        # We go into the 'login_token' key in our dictionary. The value we save here to
        # this variable is our authentication key.
        login_token = result_json['login_token']

        # We have the function return the key so this function can be called wherever we need
        # the key.
        return login_token

    def logout(self):
        os.remove(os.path.join(CURRENT_DIR, "sav", "credentials.pickle"))

def login_interface():
    """
    Called to display menus and options for logging in
    """
    print("\nPlease enter your Docket Alarm username and press ENTER.\n")
    input_username = input()
    menus.clear()
    print("\nPlease enter your Docket Alarm password and press ENTER\n(This will be stored securely on your local machine)\n")
    input_password = stdiomask.getpass(mask="*", prompt="")
    menus.clear()
    # This is the endpoint for logging in to Docket Alarm from the API.
    login_url = "https://www.docketalarm.com/api/v1/login/"
    # The data we will send to the endpoint with our post request will be
    # our Docket Alarm username and password.
    data = {
        'username':input_username,
        'password':input_password,
        }
    # We save the response to a variable. The response is a json object containing
    # our authentication key iside the json key named 'login_token'
    result = requests.post(login_url, data=data)
    # Calling the .json() method on the result turns it into a python dictionary we
    # can work with natively.
    result_json = result.json()
    # Checks to see if the login was a success...
    if result_json['success'] != True:
        # If it was not a sucess, we let the user know...
        print(result_json['error'])
        input()
        # And prompt the user to log in again.
        login_interface()
    else:
        # If the login is a success, we store the user info in a pickle file to be accessed later.
        # That way, the program can log in automatically every time the script is run afterwards.
        store_user_info_locally(input_username, input_password)


    
    




# Built-in Modules
import tkinter
from tkinter import filedialog 
import os

CURRENT_DIR = os.path.dirname(__file__)

FILEBROWSER_CSV_INITIAL_DIR = os.path.join(CURRENT_DIR, 'csv')

def browseCSVFiles():
    """
    Opens a file browser window in the 'csv' directory and prompts the
    user to select a csv file.
    """

    # If this code is not included, TKinter will try to open a blank window that can't be closed.
    # Including it makes sure the root window is not shown, since this is a command line application
    # and there is no root. We only use this small gui menu for file browsing.
    root = tkinter.Tk()
    root.withdraw()


    # This code has the file browser come up in focus.
    root.overrideredirect(True)
    root.geometry('0x0+0+0')
    root.deiconify()
    root.lift()
    root.focus_force()

    # Opens the file browser GUI.
    # The initialdir argument specifies which folder the browser will open to by default.
    # Title allows you to display text instructions on the box.
    # filetypes allows you to specify what filetypes the user is allowed to select,
    # the first filetype listed is the default.
    root.filename = filedialog.askopenfilename(initialdir = FILEBROWSER_CSV_INITIAL_DIR, 
                                          title = "Select a File", 
                                          filetypes = (("Comma Separated Values files", 
                                                        "*.csv*"), 
                                                       ("all files", 
                                                        "*.*")))


    return root.filename

def browseDirectories(initialdir):
    """
    Opens a file browser in the 'pdf-output' directory that looks for directories.
    Takes 1 argument: the relative path to the directory you want the browser to start in.
    """

    # Gets rid of the blank tkinter root window that will show up alongside the script if you don't include this code.
    root = tkinter.Tk()
    root.withdraw()

    # Opens the file browser and returns the path to the directory the user selected as a string.
    root.directory = filedialog.askdirectory(initialdir = os.path.join(CURRENT_DIR, initialdir), title ="Choose PDF Output Directory")
    return root.directory




# These are your global variables. They are important to change if you want
# to repurpose this code to make API calls based on a different spreadsheet,
# or if you want to search within a different court.

# Where is the default path to the input spreadisheet?
spreadsheet = "input.csv"

# Do you want a cached version of the docket, or a new one?
# (UnCached versions may be more expensive.)
isCached = True

# Do you want to use the GUI version of the program, or the command-line-interface version?
isGUI = False

# This global variable also should not be changed unless you know for sure it suits
# your use case. In some situations, the case numbers provided may not come in a
# format that Docket Alarm recognizes. If this is set to True, then the program
# will attempt to reformat those case numbers.
formatCaseNos = False




import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
import menus
import config

def run():
    menus.welcome()


if __name__ == '__main__':
    if config.isGUI == False:
        # If isGUI is set to False in config/config.py, then the program will run in the command line when this file is executed.
        run()
    if config.isGUI == True:
        # If isGUI is set to True in config/config.py, Then the program will open with an experimental GUI. (This is not reccomended as of yet.)
        menus.gui_run()



from setuptools import find_packages, setup
import pathlib

with open("README.md") as f:
    long_description = f.read()

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(name='docket-alarm-api-bulk-download',
    version='0.0.1',
    description="Uses the Docket Alarm API to pull state court cases in bulk",
    long_description=README,
    long_description_content_type='text/markdown',
    description_content_type='text/markdown',
    author='Ryan Fitzpatrick',
    author_email='ryanfitz514@gmail.com',
    url="https://github.com/DocketAlarm/state-court-bulk-docket-pull",
    license="Apache",
    keywords=['DOCKET', 'ALARM', 'API', 'BULK', 'DOWNLOAD', 'COURT', 'COURTS', 'PACER', 'FASTCASE'],
    packages=["docket_alarm_api_bulk_download"],
    package_dir = {'docket_alarm_api_bulk_download':'docket_alarm_api_bulk_download'},
    package_data = {'docket_alarm_api_bulk_download':['docket_alarm_api_bulk_download/csv/input.sample.csv','docket_alarm_api_bulk_download/docs/.gitkeep','docket_alarm_api_bulk_download/json-output/.gitkeep','docket_alarm_api_bulk_download/log/.gitkeep','docket_alarm_api_bulk_download/pdf-output/.gitkeep', 'docket_alarm_api_bulk_download/sav/.gitkeep']},
    include_package_data=True,
    install_requires=[
'stdiomask',
'colorama',
'progress',
'pandas',
'tqdm',
'requests',
'retrying',
'PyPDF2',
'PySimpleGUI',
'openpyxl',
    ],
    entry_points={
        'console_scripts':['docket-alarm-api-bulk-download=docket_alarm_api_bulk_download.__main__:run'],
    }
    )


import requests
import get_json

def search_direct(docketnum, court):
    """ Takes in the name of a party and the docket number as a parameter,
        returns the Docket Alarm search results. You can make calls to the
        /getdocket endpoint with these results to get more detailed information
        on the docket you are looking for.
    """
    searchdirect_url = "https://www.docketalarm.com/api/v1/searchdirect/"

    data = {
        'login_token':get_json.authenticate(),
        'client_matter':"",
        # 'party_name':party_name,
        'docketnum':docketnum,
        'court': court,
        # 'case_type':'CF',

    }
    
    result = requests.post(searchdirect_url, data)

    result_json = result.json()

    return result_json

def search_pacer(docketnum, court):
    url = "https://www.docketalarm.com/api/v1/searchpacer/"

    data = {
        'login_token':get_json.authenticate(),
        'client_matter':"",
        # 'party_name':party_name,
        'docket_num':docketnum,
        'court_region': court,
        # 'case_type':'CF',

    }
    
    result = requests.get(url, data)

    result_json = result.json()

    return result_json